/*
Example MQTT-switch-relay-node with 4 buttons and 4 leds

    connects to an MQTT server

    publishes “hello world" to the topic

 

    sends a specific “on"/“off" to the topic a specific button is pressed

    multiple arduino’s with same generic sketch can run parallel to each other

    multiple arduino’s need each to have a unique ip-addres, unique mac address and unique MQTT client-ID

    tested on arduino-uno with W5100 ethernet shield

    Ethernet Shield W5100 uses pins 4,10,11,12,13

    availbale digital pins: 1,2,3,5,6,7,8,9
    */

//------------------------------------------------------------------------------

#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h>


// Set led variables to Arduino digital pins
const byte relay1 = 2;
const byte relay2 = 3;
const byte relay3 = 5; // pin 4 used by ethernet shield
const byte relay4 = 6;
const byte relay5 = 7;
const byte relay6 = 8;
const byte relay7 = 9;
const byte relay8 = A0;



//---------------------------------------------------------------------------

// Arduino MAC address is on a sticker on your Ethernet shield
// must be unique for every node in same network
// To make a new unique address change last letter

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xEE };

// Unique static IP address of this Arduino - change to adapt to your network
IPAddress ip(192,168,0,238);

// IP Address of your MQTT broker - change to adapt to your network
byte server[] = { 192, 168, 0, 20 };



void callback(char*, byte*, unsigned int);
void reconnect();


// Initiate instances -----------------------------------

EthernetClient ethClient;
PubSubClient client(server, 1883, callback, ethClient);



//-------------------------------------------------------

void setup()

{

// setup relays
    pinMode(relay1, OUTPUT);
    pinMode(relay2, OUTPUT);
    pinMode(relay3, OUTPUT);
    pinMode(relay4, OUTPUT);
    pinMode(relay5, OUTPUT);
    pinMode(relay6, OUTPUT);
    pinMode(relay7, OUTPUT);
    pinMode(relay8, OUTPUT);







// Setup serial connection
Serial.begin(9600);

// Setup ethernet connection to MQTT broker
Ethernet.begin(mac);
    if (client.connect("arduino-ip-238")) { // change as desired - clientname must be unique for MQTT broker
       // client.publish("status/relay","hello world - here arduino ip 239");
        Serial.println("connected");
        client.subscribe("control/relay");
    }
}

//----------------------------------------------

void loop()
{

    if (!client.connected()) {
        reconnect();
    }
    client.loop();
}


void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");

    // Attempt to connect
   if (client.connect("arduino-ip-238")) {
      Serial.println("connected"); 
      // Subscribe or resubscribe to a topic
      // You can subscribe to more topics (to control more outputs)
      client.subscribe("control/relay"); 

    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds time after  retrying
      delay(5000);
    }
  }
}


void callback(char* topic, byte* payload, unsigned int length) {
    // handle message arrived
    String content="";
    char character;

    for (unsigned int num=0;num<length;num++) {
        character = payload[num];
        content.concat(character);
    }

    Serial.print(topic);
    Serial.print(" payload: ");
    Serial.println(content);

    unsigned long time = millis();

    if (content == "1on") {
        Serial.print("1 on - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay1,HIGH);
        Serial.print("1 on - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","1on");

    }

    if (content == "1off") {
        Serial.print("1 off - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay1, LOW);
        Serial.print("1 off - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","1off");

    }

    if (content == "2on") {
        Serial.print("2 on - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay2, HIGH);
        Serial.print("2 on - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","2on");

    }

    if (content == "2off") {
        Serial.print("2 off - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay2, LOW);
        Serial.print("2 off - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","2off");
    }

    if (content == "3on") {
        Serial.print("3 on - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay3, HIGH);
        Serial.print("3 on - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","3on");
    }

    if (content == "3off") {
        Serial.print("3 off - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay3, LOW);
        Serial.print("3 off - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","3off");

    }

    if (content == "4on") {
        Serial.print("4 on - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay4, HIGH);
        Serial.print("4 on - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","4on");

    }

    if (content == "4off") {
        Serial.print("4 off - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay4, LOW);
        Serial.print("4 off - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","4off");

    }

    if (content == "5on") {
        Serial.print("5 on - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay5, HIGH);
        Serial.print("5 on - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","5on");

    }

    if (content == "5off") {
        Serial.print("5 off - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay5, LOW);
        Serial.print("5 off - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","5off");

    }

    if (content == "6on") {
        Serial.print("6 on - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay6, HIGH);
        Serial.print("6 on - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","6on");

    }

    if (content == "6off") {
        Serial.print("6 off - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay6, LOW);
        Serial.print("6 off - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","6off");

    }

    if (content == "7on") {
        Serial.print("7 on - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay7, HIGH);
        Serial.print("7 on - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","7on");

    }

    if (content == "7off") {
        Serial.print("7 off - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay7, LOW);
        Serial.print("7 off - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","7off");

    }

    if (content == "8on") {
        Serial.print("8 on - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay8, HIGH);
        Serial.print("8 on - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","8on");

    }

    if (content == "8off") {
        Serial.print("8 off - time before ");
        Serial.println(millis()-time);
        digitalWrite(relay8, LOW);
        Serial.print("8 off - time after ");
        Serial.println(millis()-time);
        client.publish("state/relay","8off");

    }

    
}

// End of sketch ---------------------------------
